
from OFS import Folder
from Globals import HTMLFile, MessageDialog
from AccessControl import getSecurityManager

import os
import re
import string
import tempfile
import types

import StringIO

import zipfile

# check zip file names for:
filenamepattern=re.compile(r'[\w\_\-\.]+$')

MAXZIPFILESIZE=4*1024*1024 # 4MB should be enough.
MINZIPFILESIZE=22


manage_addZipFolderForm=HTMLFile('zipfolderAdd', globals())

def manage_addZipFolder(self, id, title='',
                          file=None,
                          REQUEST=None):
    """Add a new ZipFolder object with id *id*, uploading file.

    """
    ob=ZipFolder()
    ob.id=str(id)
    ob.title=title
    self._setObject(id, ob)
    ob=self._getOb(id)

    checkPermission=getSecurityManager().checkPermission    

    if file is not None:
        if not checkPermission('Add Documents, Images, and Files', ob):
            raise 'Unauthorized', (
                  'You are not authorized to add DTML Documents.'
                  )
        ob.manage_upload(file)

    if REQUEST is not None:
        return self.manage_main(self, REQUEST, update_menu=1)



class ZipFolder(Folder.Folder):
    """Ein Ordner mit einer Erweiterung, um seinen Inhalt
    und seine Attribute in Form einer ZIP-Datei hoch- und
    runterladen zu knnen."""

    meta_type="ZipFolder"

    __ac_permissions__=(
        ('View management screens',
         ('manage_uploadForm',)),
        ('Add Documents, Images, and Files',
         ('manage_upload','manage_uploadForm')),
        ('Delete objects',
         ('manage_upload','manage_uploadForm')))

    
    manage_options=(Folder.Folder.manage_options+
                    ({ "label": "Upload",
                       "action": "manage_uploadForm"},)
                    )


    manage_uploadForm=HTMLFile('zipfolderUpload', globals())
    

    def manage_upload(self,file='',REQUEST=None, RESPONSE=None):
        """Accept a file and load it up"""

        if isinstance(file,types.StringType):
            if REQUEST: return MessageDialog(
                title  ='Success!',
                message='But no ZIP file.',
                action ='manage_main')
            return # anyway
        
        
        # Voraussetzungen hier:
        # File hat eine Maximalgre
        file.seek(0,2)
        fsize=file.tell()
        file.seek(0,0)
        if fsize > MAXZIPFILESIZE:
            raise RuntimeError,'ZIP file is too big'

        if fsize == 0:
            if RESPONSE:
                RESPONSE.redirect('manage_main')
            return # anyway

        if fsize < MINZIPFILESIZE:
            raise RuntimeError,'ZIP file is too small'

        # Write file
        tmpfilename=tempfile.mktemp()
        tmpfile=open(tmpfilename,'w+')

        try:
            tmpfile.write(file.read())
        finally:
            tmpfile.close()

        try:
            zf=zipfile.ZipFile(tmpfilename,'r')
            
            # remove old documents

            self._remove_objects(('File','Image','DTML Document'))
            self._check_zip(zf)

            for name in zf.namelist():

                self._add_file(zf,name)
                
            if REQUEST: return MessageDialog(
                title  ='Success!',
                message='Your changes have been saved',
                action ='manage_main')

        finally:
            os.unlink(tmpfilename)

            

    def _remove_objects(self,metatypes):
        """Removes old objects from the folder"""
        
        removelist=self.objectIds(metatypes)
        self.manage_delObjects(ids=removelist)

    def _add_file(self,zipfile,name):

        # may cause exceptions: duplicate basenames et al!
        basename=os.path.basename(name)
        
        if not filenamepattern.match(basename):
            return # skip ugly files
        
        suffix=os.path.splitext(basename)[1]
        
        if string.lower(suffix) in ['.htm', '.html']:
            sf=StringIO.StringIO(self._edit_html(zipfile.read(name)))
            
            self.manage_addDTMLDocument(id=basename, title='', file=sf)
            sf.close()
            
        elif string.lower(suffix) in ['.gif', '.jpg', '.jpeg', '.png']:
            sf=StringIO.StringIO(zipfile.read(name))
            self.manage_addImage(id=basename, title='', file=sf)
            sf.close()
        else:
            self._add_other_file(zipfile,name,basename)
            

    def _check_zip(self,zipfile):
        # a hook to check zipfile before import loop
        # maybe: Look for a special file in the zip...
        pass
    
    
    def _edit_html(self, data):
        # a hook for filters to edit the imported html files
        return data

    def _add_other_file(self,zipfile,name,basename):
        # Hook for other known file types, default: add file objects
        sf=StringIO.StringIO(zipfile.read(name))
        self.manage_addFile(id=basename, title='', file=sf)
        sf.close()

